@@ -11,10 +11,6 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
  * 4. Neither the name of the University nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
@@ -438,7 +434,7 @@ ip6_stats(u_long off, const char *name, int af1 __unused, int proto __unused)
 	printf("\t\t%ju two or more ext mbuf\n",
 	    (uintmax_t)ip6stat.ip6s_mext2m);
 	p(ip6s_exthdrtoolong,
-	    "\t%ju packet%s whose headers are not continuous\n");
+	    "\t%ju packet%s whose headers are not contiguous\n");
 	p(ip6s_nogif, "\t%ju tunneling packet%s that can't find gif\n");
 	p(ip6s_toomanyhdr,
 	    "\t%ju packet%s discarded because of too many headers\n");
@@ -512,8 +508,6 @@ ip6_stats(u_long off, const char *name, int af1 __unused, int proto __unused)
 		}
 	}
 
-	p1a(ip6s_forward_cachehit, "\t%ju forward cache hit\n");
-	p1a(ip6s_forward_cachemiss, "\t%ju forward cache miss\n");
 	printf("\tSource addresses selection rule applied:\n");
 	for (i = 0; i < 16; i++) {
 		if (ip6stat.ip6s_sources_rule[i])
@@ -713,8 +707,8 @@ static	const char *icmp6names[] = {
 	"echo",
 	"echo reply",
 	"multicast listener query",
-	"multicast listener report",
-	"multicast listener done",
+	"MLDv1 listener report",
+	"MLDv1 listener done",
 	"router solicitation",
 	"router advertisement",
 	"neighbor solicitation",
@@ -725,7 +719,7 @@ static	const char *icmp6names[] = {
 	"node information reply",
 	"inverse neighbor solicitation",
 	"inverse neighbor advertisement",
-	"#143",
+	"MLDv2 listener report",
 	"#144",
 	"#145",
 	"#146",
